/* Sistemas Integrados TECNISA
   Compras e Suprimentos

Nome: 8951_SUPSTPS_GERA_SOLICITA_OL.sql

Descrio:
Gera solicitao a partir da consolidao do operador logstico.

P_CDFILIOPELOG - Operador Logstico.
P_NRSEQUCONSOL - N da Consolidao
P_CDFILLANC    - Filial de Lanamento

Criao:
16/02/2007 Marcelo Alexsandro Souza Silva
*/

IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'GERA_SOLICITA_OL'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)
   DROP PROCEDURE GERA_SOLICITA_OL;
GO

CREATE PROCEDURE GERA_SOLICITA_OL(
   @P_CDFILIOPELOG VARCHAR(4),
   @P_NRSEQUCONSOL VARCHAR(10),
   @P_CDFILLANC    VARCHAR(4),
   @P_CDOPERLANC   VARCHAR(12),
   @P_ROWSAFFECT   VARCHAR(20) OUTPUT) AS

DECLARE @V_PRXSOLICMP   VARCHAR(20);
DECLARE @V_CDFILCENT    VARCHAR(4);
DECLARE @V_PRODUTO      VARCHAR(10);
DECLARE @V_IDUTAPROVSOL VARCHAR(1);
DECLARE @V_IDSOLIAPROV  VARCHAR(1);
DECLARE @V_IDPERALTSAPR VARCHAR(1);
DECLARE @W_A            VARCHAR(1);
DECLARE @W_S            VARCHAR(1);
DECLARE @W_N            VARCHAR(1);
DECLARE @W_C            VARCHAR(1);
DECLARE @VSOLICOMP VARCHAR(200);

DECLARE @V_ESTOQUERESTANTE      DOUBLE PRECISION;
DECLARE @V_QTDEITEMSOLI         DOUBLE PRECISION;

DECLARE   @W_ZERO               INT;
DECLARE   @W_UM                 INT;
DECLARE   @W_ONZE               INT;
DECLARE   @W_DEZ                INT;
DECLARE   @W_TRES               INT;

DECLARE @V_ITEM_CDPRODUTO       VARCHAR(10);
DECLARE @V_ITEM_DTENTRCONSOL    DATETIME;
DECLARE @V_ITEM_QTENTRCONSOL    DOUBLE PRECISION;
DECLARE @V_ITEM_VRARREDCMP      DOUBLE PRECISION;
DECLARE @V_ITEM_QTESTUTILCONSOL DOUBLE PRECISION;

DECLARE @V_DTPROCESS            DATETIME;

BEGIN
   SET @W_A            = 'A';
   SET @W_S            = 'S';
   SET @W_N            = 'N';
   SET @W_C            = 'C';
   SET @W_ZERO         = 0;
   SET @W_UM           = 1;
   SET @W_ONZE         = 11;
   SET @W_DEZ          = 10;
   SET @W_TRES         = 3;
   SET @V_IDUTAPROVSOL = '';
   SET @V_PRXSOLICMP   = '';
   SET @V_CDFILCENT    = '';
   SET @V_IDSOLIAPROV  = '';
   SET @V_IDPERALTSAPR = '';

   SET @VSOLICOMP      = 'SOLICOMP'+@P_CDFILIOPELOG+@P_CDFILLANC;
   EXEC NOVO_CODIGO @VSOLICOMP, @P_SEQUENCIAL = @V_PRXSOLICMP OUTPUT, @P_QTDE = 1;
   SET @V_PRXSOLICMP = SUBSTRING(@V_PRXSOLICMP,11,10);

   SELECT
     @V_IDUTAPROVSOL = MAX(IDUTAPROVSOL),
     @V_IDPERALTSAPR = MAX(SUBSTRING(ISNULL(IDPERALTSAPR, @W_N+@W_N+@W_N), @W_TRES, @W_UM))
   FROM
      PARCMPSUP
   WHERE
      CDFILIAL = @P_CDFILIOPELOG;

   SELECT
      @V_DTPROCESS = CASE ISNULL(MAX(P.IDUTILDTPROC), @W_N)
                        WHEN @W_S THEN ISNULL(MAX(PF.DTPROCESSA), CONVERT(DATETIME, GETDATE(),103))
                        ELSE CONVERT(DATETIME, GETDATE(),103)
                     END
   FROM
      PARAMFILIAL PF, PARAMGERAL P
   WHERE PF.CDFILIAL = @P_CDFILIOPELOG;


   IF @V_IDUTAPROVSOL = @W_S BEGIN
      IF @V_IDPERALTSAPR = @W_S BEGIN
         SET @V_IDSOLIAPROV = @W_S;
      END
      ELSE BEGIN
         SET @V_IDSOLIAPROV = @W_N;
      END;
   END
   ELSE BEGIN
      SET @V_IDSOLIAPROV = @W_S;
   END;

   SELECT
      @V_CDFILCENT = MAX(ISNULL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL))
   FROM
      FILIAL
   WHERE
      FILIAL.CDFILIAL = @P_CDFILIOPELOG;

   INSERT INTO SOLICITA
     (CDFILSOLI,
      CDFILLANCA,
      NRSOLICMP,
      DTSOLCMP,
      IDTIPLAN,
      CDFILCENT,
      CDOPERLANC)
   VALUES
     (@P_CDFILIOPELOG,
      @P_CDFILLANC,
      @V_PRXSOLICMP,
      CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103), 103),
      @W_A,
      @V_CDFILCENT,
      @P_CDOPERLANC);

   INSERT INTO ITEMSOLI
     (CDFILSOLI,
      CDFILLAN,
      NRSOLICMP,
      CDPRODUTO,
      DTPROGENTR,
      QTSOLIPROD,
      IDSOLIAPROV,
      NRSEQUCONSOL)
   SELECT
      @P_CDFILIOPELOG,
      @P_CDFILLANC,
      @V_PRXSOLICMP,
      PRODCONSSOLOL.CDPRODUTO,
      ITEMCONSSOLOL.DTENTRCONSOL,
      CEILING(CAST((ITEMCONSSOLOL.QTENTRCONSOL   - ISNULL(JASOLICITA.QTDE, 0)) AS DOUBLE PRECISION) / ISNULL(PRODUTO.VRARREDCMP,1)) * ISNULL(PRODUTO.VRARREDCMP,1),
      @V_IDSOLIAPROV,
      @P_NRSEQUCONSOL
   FROM
      PRODCONSSOLOL,
      PRODUTO,      
      ITEMCONSSOLOL LEFT JOIN
                             (SELECT SUM(I.QTSOLIPROD) QTDE, I.CDFILSOLI, I.CDPRODUTO, I.DTPROGENTR
                              FROM ITEMSOLI I, CONSSOLOL
                              WHERE I.CDFILSOLI   = @P_CDFILIOPELOG
                                AND I.DTPROGENTR >= CONVERT(DATETIME, CONVERT(DOUBLE PRECISION, @V_DTPROCESS - 90), 103)
                                AND I.NRSEQUCONSOL = CONSSOLOL.NRSEQUCONSOL
                                AND ISNULL(CONSSOLOL.IDCONSPREVCONOL, 'N')  = 'S'
                              GROUP BY I.CDFILSOLI, I.CDPRODUTO, I.DTPROGENTR) JASOLICITA
                           ON JASOLICITA.CDFILSOLI = @P_CDFILIOPELOG
                          AND JASOLICITA.CDPRODUTO = ITEMCONSSOLOL.CDPRODUTO
                          AND JASOLICITA.DTPROGENTR= ITEMCONSSOLOL.DTENTRCONSOL
   WHERE PRODCONSSOLOL.NRSEQUCONSOL    = @P_NRSEQUCONSOL
     AND PRODCONSSOLOL.NRSEQUCONSOL    = ITEMCONSSOLOL.NRSEQUCONSOL
     AND PRODCONSSOLOL.CDFILSOLI       = ITEMCONSSOLOL.CDFILSOLI
     AND PRODCONSSOLOL.CDFILLAN        = ITEMCONSSOLOL.CDFILLAN
     AND PRODCONSSOLOL.CDPRODUTO       = ITEMCONSSOLOL.CDPRODUTO
     AND ISNULL(ITEMCONSSOLOL.IDENTREGAVIAOL, @W_S) = @W_S
     AND ISNULL(PRODCONSSOLOL.QTESTUTILCONSOL, @W_ZERO) <= @W_ZERO
     AND (ITEMCONSSOLOL.QTENTRCONSOL   - ISNULL(JASOLICITA.QTDE, 0)) > 0
     AND PRODUTO.CDPRODUTO             = PRODCONSSOLOL.CDPRODUTO;





   SET @V_PRODUTO = ' ';
   DECLARE C_ITEM CURSOR FOR
       SELECT
          PRODCONSSOLOL.CDPRODUTO,
          PRODCONSSOLOL.QTESTUTILCONSOL,
          ITEMCONSSOLOL.DTENTRCONSOL,
          (ITEMCONSSOLOL.QTENTRCONSOL - ISNULL(JASOLICITA.QTDE, 0)) QTENTRCONSOL,
          ISNULL(PRODUTO.VRARREDCMP, @W_ZERO) VRARREDCMP
       FROM
          PRODCONSSOLOL,
          PRODUTO,
          ITEMCONSSOLOL LEFT JOIN
                                 (SELECT SUM(I.QTSOLIPROD) QTDE, I.CDFILSOLI, I.CDPRODUTO, I.DTPROGENTR
                                  FROM ITEMSOLI I, CONSSOLOL
                                  WHERE I.CDFILSOLI   = @P_CDFILIOPELOG
                                    AND I.DTPROGENTR >= CONVERT(DATETIME, CONVERT(DOUBLE PRECISION, @V_DTPROCESS - 90), 103)
                                    AND I.NRSEQUCONSOL = CONSSOLOL.NRSEQUCONSOL
                                    AND ISNULL(CONSSOLOL.IDCONSPREVCONOL, 'N')  = 'S'
                                  GROUP BY I.CDFILSOLI, I.CDPRODUTO, I.DTPROGENTR) JASOLICITA
                               ON JASOLICITA.CDFILSOLI = @P_CDFILIOPELOG
                              AND JASOLICITA.CDPRODUTO = ITEMCONSSOLOL.CDPRODUTO
                              AND JASOLICITA.DTPROGENTR= ITEMCONSSOLOL.DTENTRCONSOL
       WHERE PRODCONSSOLOL.NRSEQUCONSOL    = @P_NRSEQUCONSOL
         AND PRODCONSSOLOL.CDPRODUTO       = PRODUTO.CDPRODUTO
         AND PRODCONSSOLOL.NRSEQUCONSOL    = ITEMCONSSOLOL.NRSEQUCONSOL
         AND PRODCONSSOLOL.CDFILSOLI       = ITEMCONSSOLOL.CDFILSOLI
         AND PRODCONSSOLOL.CDFILLAN        = ITEMCONSSOLOL.CDFILLAN
         AND PRODCONSSOLOL.CDPRODUTO       = ITEMCONSSOLOL.CDPRODUTO
         AND ISNULL(ITEMCONSSOLOL.IDENTREGAVIAOL, @W_S) = @W_S
         AND ISNULL(PRODCONSSOLOL.QTESTUTILCONSOL, @W_ZERO) > @W_ZERO
         AND (ITEMCONSSOLOL.QTENTRCONSOL   - ISNULL(JASOLICITA.QTDE, 0)) > 0
       ORDER BY
          PRODCONSSOLOL.CDPRODUTO,
          ITEMCONSSOLOL.DTENTRCONSOL

   OPEN C_ITEM;
   FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDPRODUTO, @V_ITEM_QTESTUTILCONSOL,
                               @V_ITEM_DTENTRCONSOL, @V_ITEM_QTENTRCONSOL,
                               @V_ITEM_VRARREDCMP;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      IF @V_PRODUTO <> @V_ITEM_CDPRODUTO BEGIN
         SET @V_ESTOQUERESTANTE = @V_ITEM_QTESTUTILCONSOL;
         SET @V_PRODUTO         = @V_ITEM_CDPRODUTO;
      END;
      IF @V_ESTOQUERESTANTE >= @V_ITEM_QTENTRCONSOL BEGIN
         SET @V_ESTOQUERESTANTE = @V_ESTOQUERESTANTE - @V_ITEM_QTENTRCONSOL;
         INSERT INTO MOVOPSOL
            (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, CDPRODUTO, DTENTROPER, DTOPERSOL,
             CDOPERADOR,  DSJUSTOPER,  QTANTOPER, QTDEPOPER, IDOPERAMOVS)
         SELECT
            @P_CDFILIOPELOG,
            @P_CDFILLANC,
            @V_PRXSOLICMP,
            @V_ITEM_CDPRODUTO,
            @V_ITEM_DTENTRCONSOL,
            DTJUSTCONSOL,
            CDOPERADOR,
            DSJUSTCONSOL,
            @V_ITEM_QTENTRCONSOL,
            @W_ZERO,
            @W_A
         FROM
            JUSTCONSSOLOL
         WHERE NRSEQUCONSOL = @P_NRSEQUCONSOL
           AND DTENTJUSTCON IS NULL
           AND CDPRODUTO    = @V_ITEM_CDPRODUTO
           AND DTJUSTCONSOL = (SELECT MAX(J.DTJUSTCONSOL)
                               FROM JUSTCONSSOLOL J
                               WHERE J.NRSEQUCONSOL = @P_NRSEQUCONSOL
                                 AND J.CDPRODUTO    = @V_ITEM_CDPRODUTO
                                 AND J.DTENTJUSTCON IS NULL);
      END
      ELSE BEGIN
         SET @V_QTDEITEMSOLI = @V_ITEM_QTENTRCONSOL - @V_ESTOQUERESTANTE;
         SET @V_ESTOQUERESTANTE = 0;
         IF (((CAST(@V_QTDEITEMSOLI AS DOUBLE PRECISION) / @V_ITEM_VRARREDCMP)  - (ROUND(@V_QTDEITEMSOLI / @V_ITEM_VRARREDCMP, 0, 1))) <> 0) AND @V_ITEM_VRARREDCMP > 0
         BEGIN
            SET @V_QTDEITEMSOLI = CEILING(CAST(@V_QTDEITEMSOLI AS DOUBLE PRECISION) / @V_ITEM_VRARREDCMP) * @V_ITEM_VRARREDCMP;
         END;
         INSERT INTO ITEMSOLI
           (CDFILSOLI,
            CDFILLAN,
            NRSOLICMP,
            CDPRODUTO,
            DTPROGENTR,
            QTSOLIPROD,
            IDSOLIAPROV,
            NRSEQUCONSOL)
         VALUES
            (@P_CDFILIOPELOG,
             @P_CDFILLANC,
             @V_PRXSOLICMP,
             @V_ITEM_CDPRODUTO,
             @V_ITEM_DTENTRCONSOL,
             @V_QTDEITEMSOLI,
             @V_IDSOLIAPROV,
             @P_NRSEQUCONSOL);

         IF @V_ITEM_QTENTRCONSOL <> @V_QTDEITEMSOLI BEGIN
            INSERT INTO MOVOPSOL
               (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, CDPRODUTO, DTENTROPER, DTOPERSOL,
                CDOPERADOR,  DSJUSTOPER,  QTANTOPER, QTDEPOPER, IDOPERAMOVS)
            SELECT
               @P_CDFILIOPELOG,
               @P_CDFILLANC,
               @V_PRXSOLICMP,
               @V_ITEM_CDPRODUTO,
               @V_ITEM_DTENTRCONSOL,
               DTJUSTCONSOL,
               CDOPERADOR,
               DSJUSTCONSOL,
               @V_ITEM_QTENTRCONSOL,
               @V_QTDEITEMSOLI,
               @W_A
            FROM
               JUSTCONSSOLOL
            WHERE NRSEQUCONSOL = @P_NRSEQUCONSOL
              AND DTENTJUSTCON IS NULL
              AND CDPRODUTO    = @V_ITEM_CDPRODUTO
              AND DTJUSTCONSOL = (SELECT MAX(J.DTJUSTCONSOL)
                                  FROM JUSTCONSSOLOL J
                                  WHERE J.NRSEQUCONSOL = @P_NRSEQUCONSOL
                                    AND J.CDPRODUTO    = @V_ITEM_CDPRODUTO
                                    AND J.DTENTJUSTCON IS NULL);
         END;
      END;
      FETCH NEXT FROM C_ITEM INTO @V_ITEM_CDPRODUTO, @V_ITEM_QTESTUTILCONSOL,
                                  @V_ITEM_DTENTRCONSOL, @V_ITEM_QTENTRCONSOL,
                                  @V_ITEM_VRARREDCMP;

   END;
   UPDATE CONSSOLOL
      SET IDSITUCONSSOL  = @W_C,
          DTCONCONSSOLOL = GETDATE()
   WHERE
      CONSSOLOL.NRSEQUCONSOL = @P_NRSEQUCONSOL;

   SET @P_ROWSAFFECT = @V_PRXSOLICMP;

   UPDATE ITEMSOLI
      SET CDFILIOPELOG = NULL,
          NRSEQUCONSOL = NULL
   WHERE
      EXISTS (SELECT @W_UM
              FROM RELACONSOLOL, ITEMCONSSOLOL
              WHERE RELACONSOLOL.CDFILSOLI       = ITEMSOLI.CDFILSOLI
                AND RELACONSOLOL.CDFILLAN        = ITEMSOLI.CDFILLAN
                AND RELACONSOLOL.NRSOLICMP       = ITEMSOLI.NRSOLICMP
                AND RELACONSOLOL.CDPRODUTO       = ITEMSOLI.CDPRODUTO
                AND RELACONSOLOL.DTPROGENTR      = ITEMSOLI.DTPROGENTR
                AND ITEMCONSSOLOL.NRSEQUCONSOL   = @P_NRSEQUCONSOL
                AND ITEMCONSSOLOL.CDFILSOLI      = @P_CDFILIOPELOG
                AND ISNULL(ITEMCONSSOLOL.IDENTREGAVIAOL, @W_S) = @W_N
                AND RELACONSOLOL.NRSEQUCONSOL    = ITEMCONSSOLOL.NRSEQUCONSOL
                AND RELACONSOLOL.CDFILSOLCONS    = ITEMCONSSOLOL.CDFILSOLI
                AND RELACONSOLOL.CDFILLANCONS    = ITEMCONSSOLOL.CDFILLAN
                AND RELACONSOLOL.CDPRODUTOCONS   = ITEMCONSSOLOL.CDPRODUTO
                AND RELACONSOLOL.DTENTRCONSOL    = ITEMCONSSOLOL.DTENTRCONSOL);

   INSERT INTO MOVOPSOL
   (CDFILSOLMOV, CDFILLANMOV, NRSOLIMOV, CDPRODUTO, DTENTROPER, DTOPERSOL,
    CDOPERADOR,  DSJUSTOPER,  QTANTOPER, QTDEPOPER, IDOPERAMOVS)
   SELECT
      @P_CDFILIOPELOG,
      @P_CDFILLANC,
      @V_PRXSOLICMP,
      CDPRODUTO,
      DTENTJUSTCON,
      DTJUSTCONSOL,
      CDOPERADOR,
      DSJUSTCONSOL,
      QTANTESTUTIL,
      QTDEPESTUTIL,
      @W_A
   FROM
      JUSTCONSSOLOl
   WHERE NRSEQUCONSOL = @P_NRSEQUCONSOL
     AND DTENTJUSTCON IS NOT NULL;

   DELETE FROM
      RELACONSOLOL
   WHERE
      EXISTS (SELECT @W_UM
              FROM ITEMCONSSOLOL
              WHERE ITEMCONSSOLOL.NRSEQUCONSOL   = @P_NRSEQUCONSOL
                AND ITEMCONSSOLOL.CDFILSOLI      = @P_CDFILIOPELOG
                AND RELACONSOLOL.NRSEQUCONSOL    = ITEMCONSSOLOL.NRSEQUCONSOL
                AND RELACONSOLOL.CDFILSOLCONS    = ITEMCONSSOLOL.CDFILSOLI
                AND RELACONSOLOL.CDFILLANCONS    = ITEMCONSSOLOL.CDFILLAN
                AND RELACONSOLOL.CDPRODUTOCONS   = ITEMCONSSOLOL.CDPRODUTO
                AND RELACONSOLOL.DTENTRCONSOL    = ITEMCONSSOLOL.DTENTRCONSOL
              );
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'GERA_SOLICITA_OL','4.11.081';
GO
